################################# LICENSE ##################################
# Copyright (c) 2009, South African Astronomical Observatory (SAAO)        #
# All rights reserved.                                                     #
#                                                                          #
# Redistribution and use in source and binary forms, with or without       #
# modification, are permitted provided that the following conditions       #
# are met:                                                                 #
#                                                                          #
#     * Redistributions of source code must retain the above copyright     #
#       notice, this list of conditions and the following disclaimer.      #
#     * Redistributions in binary form must reproduce the above copyright  #
#       notice, this list of conditions and the following disclaimer       #
#       in the documentation and/or other materials provided with the      #
#       distribution.                                                      #
#     * Neither the name of the South African Astronomical Observatory     #
#       (SAAO) nor the names of its contributors may be used to endorse    #
#       or promote products derived from this software without specific    #
#       prior written permission.                                          #
#                                                                          #
# THIS SOFTWARE IS PROVIDED BY THE SAAO ''AS IS'' AND ANY EXPRESS OR       #
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           #
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   #
# DISCLAIMED. IN NO EVENT SHALL THE SAAO BE LIABLE FOR ANY                 #
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       #
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS  #
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)    #
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,      #
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          #
# POSSIBILITY OF SUCH DAMAGE.                                              #
############################################################################

"""SALTFPSKYRING is a tool to remove skylines from images. Using T. Williams code.  The code assumes all the files are in the same directory. 

Updates:

20100917
    * First wrote the code
"""

# Ensure python 2.5 compatibility
from __future__ import with_statement

import os
import sys
import numpy as np
#import pyfits

from pyraf import iraf
from pyraf.iraf import pysalt
import saltsafekey
import saltsafeio
import fpsafeio
from saltsafelog import logging

from fortranfp import skyring_wrapper
debug=True

def saltfpskyring(axc,ayc,arad,rxc,ryc,pmin,pmax,plottype,image,outfile,logfile,verbose):  
    """Sky subtracts Fabry-Perot images"""

# default parameter values are set up in the pyraf .par file. 

    plottype = str(plottype)
    plottype = plottype.strip()
    if plottype == 'xwindow':
        plottype = '/xw'
    else:
        plottype = '/ps'

    pathin = os.path.dirname(image)
    basein = os.path.basename(image)
    pathlog = os.path.dirname(logfile)
    baselog = os.path.basename(logfile)
    baseout = os.path.basename(outfile)
    pathout = os.path.dirname(outfile) 

# forcing logfiles to be created in the same directory as the input data

    if len(pathin) > 0:
        pathlog = pathin

# start log now that all parameter are set up          

    with logging(logfile, debug) as log:

# Some basic checks, many tests are done in the FORTRAN code itself
# is the input file specified?
        saltsafeio.filedefined('Input',image)

# if the input file is a list, throw an error
        if basein[0] == '@':
            raise SaltIOError(basein + ' list input instead of a file' )

# if the input file is a file, does it exist?

        if basein[0] != '@':
            saltsafeio.fileexists(image)
            infile = image

# is the proposed output file specified?

        saltsafeio.filedefined('Output',outfile)

# check if proposed output file exists, remove if it does exist
        if baseout[0] != '@':

            testout = outfile
            testout = testout.strip()
            if os.path.isfile(testout):
                 saltsafeio.delete(testout)
            print image, testout

# note that logfiles are forced to be in the same directory as the INPUT data

# If all looks OK, run the FORTRAN code            

        skyring_wrapper.skyring(axc, ayc,arad,rxc,ryc,infile,pmin, pmax, plottype, outfile)

    
# -----------------------------------------------------------
# main code

parfile = iraf.osfn("saltfp$saltfpskyring.par")
t = iraf.IrafTaskFactory(taskname="saltfpskyring",value=parfile,function=saltfpskyring,pkgname='saltfp')
